/* This file is part of Cloudy and is copyright (C)1978-2014 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*lines_lv1_k_zn place lines of elements potassium and heavier into lines storage stack */
#include "cddefines.h"
#include "cddrive.h"
#include "coolheavy.h"
#include "ca.h"
#include "fe.h"
#include "dense.h"
#include "phycon.h"
#include "radius.h"
#include "taulines.h"
#include "trace.h"
#include "atom_feii.h"
#include "lines.h"
#include "lines_service.h"
#include "iso.h"

void lines_lv1_k_zn(void)
{
	long int i;

	double eff, fela;

	DEBUG_ENTRY( "lines_lv1_k_zn()" );

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, "   lines_lv1_k_zn called\n" );
	}


	/* bob Rubin's UV line
	 * f2 = dense.xIonDense(26,4)*sexp(50 764./te)*0.45*cdsqte/6.*7.01e-12
	 * call linadd( f2 , 2837 , 'BobR' , 'i')
	 * f2 = dense.xIonDense(26,4)*sexp(55 989./te)*0.384*cdsqte/6.*7.74e-12
	 * call linadd( f2 , 2568 , 'BobR' , 'i') */

	/* iron */

	linadd(MAX2(0.,FeII.Fe2_large_cool+FeII.Fe2_UVsimp_cool),0,"Fe2c",'c' ,
		"total of all Fe 2 cooling, both simple UV and large atom together ");

	linadd(MAX2(0.,-FeII.Fe2_large_cool-FeII.Fe2_UVsimp_cool),0,"Fe2h",'h' ,
		"total of all Fe 2 heating, both simple UV and large atom together ");

	linadd(FeII.for7,4300,"Fe 2",'i' ,
		" Fe 2 forbidden 2-1 transition from Netzer's atom ");

	PutLine(TauLines[ipTuv3],
		" 2400 in simple Wills, Netzer, Wills FeII");
	PutLine(TauLines[ipTr48],
		" 6200 in simple Wills, Netzer, Wills FeII");
	PutLine(TauLines[ipTFe16],
		" 1080 in simple Wills, Netzer, Wills FeII");
	PutLine(TauLines[ipTFe26],
		" 1500  in simple Wills, Netzer, Wills FeII");
	PutLine(TauLines[ipTFe34],
		" 11500 in simple Wills, Netzer, Wills FeII");
	PutLine(TauLines[ipTFe35],
		" 2500 in simple Wills, Netzer, Wills FeII");
	PutLine(TauLines[ipTFe46],
		" 2300 in simple Wills, Netzer, Wills FeII");
	PutLine(TauLines[ipTFe56],
		" 8900 in simple Wills, Netzer, Wills FeII");

	/* option to save all intensities predicted by large FeII atom,
	 * code is in FeIILevelPops */
	FeIIAddLines();
	/* we were called by lines, and we want to zero out Fe2SavN */
	for( long ipLo=0; ipLo < (FeII.nFeIILevel_malloc - 1); ipLo++ )
	{
		for( long ipHi=ipLo + 1; ipHi < FeII.nFeIILevel_malloc; ipHi++ )
		{
			/* only evaluate real transitions */
			TransitionProxy tr = Fe2LevN[ipFe2LevN[ipHi][ipLo]];
			if( tr.ipCont() > 0 ) 
				PutLine( tr ," Fe II emission" );
		}
	}

	/* emission bands from the model Fe II atom */
	for( i=0; i < nFeIIBands; i++ )
	{
		double SumBandInward;
		/* [i][0] is center wavelength, [i][1] and [i][2] are upper and
		 * lower bounds in Angstroms.  These are set in FeIIZero 
		 * units are erg s-1 cm-3 */
		eff = FeIISumBand(FeII_Bands[i][1],FeII_Bands[i][2],
			&SumBandInward);

		linadd(eff,FeII_Bands[i][0],"Fe2b",'i' ,
			" total Fe II emission in Fe II bands, as defined in FeII_bands.ini ");
		linadd(SumBandInward,FeII_Bands[i][0],"Inwd",'i' ,
			" inward Fe II emission in Fe II bands, as defined in FeII_bands.ini ");
	}

	// integrate the pseudo continuum of FeII emission
	if( LineSave.ipass > 0 )
	{
		// initialize
		if( nzone == 1 )
		{
			for( i=0; i < nFeIIConBins; i++ )
			{
				/* initialize arrays */
				FeII_Cont[i][1] = 0.;
				FeII_Cont[i][2] = 0.;
			}
		}

		// integrate
		for( i=0; i < nFeIIConBins; i++ )
		{
			double SumBandInward;
			/* [i][0] is total intensity in cell, [i][1] and [i][2] are lower and
			 * upper bounds in Angstroms.  these are set in FeIIZero *
			 * find total emission from large FeII atom, integrated over band */
			double TotalFeII = FeIISumBand(FeII_Cont[i][0],FeII_Cont[i+1][0],
				&SumBandInward);
			FeII_Cont[i][1] += (realnum)(SumBandInward*radius.dVeffAper);
			FeII_Cont[i][2] += (realnum)(MAX2(0.,TotalFeII-SumBandInward)*radius.dVeffAper);
			/*fprintf(ioQQQ,"DEBUG feii\t%li\t%.2e\n", i, FeII_Cont[i][0]);*/
		}
	}

	PutLine(TauLines[ipT191],
		"  anomalous Fe 2 transition at 1787, RMT 191");

	/* recombination Ka */
	if( dense.lgElmtOn[ipIRON] )
	{
		/* these lines added to outlin in metdif - following must be false
		 * fela = xLyaHeavy(nelem,nelem)*dense.xIonDense(nelem,nelem+1) */
		fela = iso_sp[ipH_LIKE][ipIRON].trans(ipH2p,ipH1s).Emis().xIntensity();
	}
	else
	{
		fela = 0.;
	}

	/* >>chng 02 jan 14, add grain fe to this sum */
	/* total intensity of K-alpha line */
	/*linadd((fe.fekcld+fe.fegrain)*1.03e-8+(fe.fekhot+fela)*1.11e-8,2,"FeKa",'i' );*/
	if( dense.lgElmtOn[ipIRON] )
	{
		lindst((fe.fekcld+fe.fegrain)*1.03e-8+(fe.fekhot+fela)*1.11e-8,1.78f,"FeKa",
			iso_sp[ipH_LIKE][ipIRON].trans(ipH2p,ipH1s).ipCont(),'i',false,
			   "total intensity of K-alpha line" );
	}

	linadd(fela*1.11e-8,2,"FeLr",'i' ,
		" recombination from fully stripped ion ");

	/* >>chng 03 aug 14, label changed from TotH to AugH to be like rest total hot iron Ka; */
	linadd((fe.fekhot+fela)*1.11e-8,2,"AugH",'i' ,
		"  Auger hot iron, assumes case b for H and He-like ");

	linadd(fe.fekcld*1.03e-8,2,"AugC",'i',
		" Auger production of cold iron, less than or 17 times ionized " );

	linadd(fe.fegrain*1.03e-8,2,"AugG",'i' ,
		" grain production of cold iron ");

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, "   lines_lv1_k_zn returns\n" );
	}
	return;
}
